/*
FRODO: a FRamework for Open/Distributed Optimization
Copyright (C) 2008, 2009, 2010, 2011  Thomas Leaute, Brammert Ottens & Radoslaw Szymanek

FRODO is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

FRODO is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.


How to contact the authors: 
<http://lia.epfl.ch/>

EPFL / IC / IIF / LIA
Batiment IN 
Station 14 
CH - 1015 Lausanne 
Switzerland
*/

package ch.epfl.lia.frodo.algorithms.maxsum;

import java.util.Arrays;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;

import ch.epfl.lia.frodo.algorithms.AbstractDCOPsolver;
import ch.epfl.lia.frodo.algorithms.AbstractSolver;
import ch.epfl.lia.frodo.algorithms.Solution;
import ch.epfl.lia.frodo.algorithms.StatsReporter;
import ch.epfl.lia.frodo.gui.DOTrenderer;
import ch.epfl.lia.frodo.solutionSpaces.Addable;

/** A solver for Max-Sum
 * @author Thomas Leaute
 * @param <V> the type used for variable values
 * @param <U> the type used for utility values
 */
public class MaxSumSolver < V extends Addable<V>, U extends Addable<U> > extends AbstractDCOPsolver< V, U, Solution<V, U> > {
	
	/** The MaxSum module that collects statistics about the solution */
	private MaxSum<V, U> module;

	/** Default constructor */
	public MaxSumSolver() {
		super ("/ch/epfl/lia/frodo/algorithms/maxsum/MaxSumAgent.xml");
	}
	
	/** Constructor
	 * @param agentConfig 	the agent configuration file
	 */
	public MaxSumSolver (Document agentConfig) {
		super (agentConfig);
	}

	/** Constructor
	 * @param agentConfig 	the agent configuration file
	 * @param useTCP 		whether to use TCP pipes
	 */
	public MaxSumSolver (Document agentConfig, boolean useTCP) {
		super (agentConfig, useTCP);
	}

	/** @see AbstractSolver#getSolGatherers() */
	@SuppressWarnings("unchecked")
	@Override
	protected List<? extends StatsReporter> getSolGatherers() {
		
		Element params = new Element ("module");
		params.setAttribute("reportStats", "false");
		params.setAttribute("DOTrenderer", DOTrenderer.class.getName());
		this.module = new MaxSum<V, U> (params, super.problem);
		
		return Arrays.asList(this.module);
	}

	/** @see AbstractSolver#buildSolution() */
	@Override
	protected Solution<V, U> buildSolution() {
		return new Solution<V, U> (super.problem.getNbrVars(), null, this.module.getOptCost(), this.module.getOptAssignments(), super.factory.getNbrMsgs(), super.factory.getMsgNbrs(), 
				super.factory.getTotalMsgSize(), super.factory.getMsgSizes(), super.factory.getNcccs(), super.factory.getTime(), null, 0);
	}

}
